#include <stdio.h>
#include <dirent.h>
#include <limits.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

struct counter {
  long reg;
  long dir;
  long blk;
  long chr;
  long fifo;
  long sock;
  long sym;
  long tot;
};

static int dopath(char *fullpath, struct counter *c); 
static void check_type(const struct stat *stat, struct counter *c);

int main(int argc, char *argv[]) {
  float perc;
  struct counter c;
  char fullpath[PATH_MAX+1];

  if (argc != 2) {
	printf("Usage: %s <starting-pathname>\n", argv[0]); 
	return 1;
  }

  /* Inizializza i valori con tutti zeri */
  memset(&c, 0, sizeof(struct counter));
  /* Copia il path iniziale */
  strncpy(fullpath, argv[1], PATH_MAX);
  /* Discende nella gerarchia partendo da 'fullpath' */
  if (dopath(fullpath, &c)) {
	return 1;
  }

  /* Stampa statistiche */
  perc = (c.tot) ? 100.0 / c.tot : 0.0;
  printf("total          = %7ld\n\n", c.tot);
  printf("regular files  = %7ld, %5.2f %%\n", c.reg, c.reg * perc);
  printf("directories    = %7ld, %5.2f %%\n", c.dir, c.dir * perc);
  printf("block special  = %7ld, %5.2f %%\n", c.blk, c.blk * perc);
  printf("char special   = %7ld, %5.2f %%\n", c.chr, c.chr * perc);
  printf("FIFOs          = %7ld, %5.2f %%\n", c.fifo, c.fifo * perc); 
  printf("symbolic links = %7ld, %5.2f %%\n", c.sym, c.sym * perc);
  printf("sockets        = %7ld, %5.2f %%\n", c.sock, c.sock * perc); 

  return 0;
}

static int dopath(char *fullpath, struct counter *c) {
  struct stat statbuf; 
  struct dirent *dirp; 
  DIR *dp; 
  char *ptr;
  int ret = 0;

  /* Ottieni le informazioni sul file */
  /* ... */

  /* Controlla se il file non e' una directory */
  if (/* non e' una directory */) {
	check_type(&statbuf, c);
	return 0;
  }

  /* 
   * Il file e' una directory, incremento il counter
   * e leggo il contenuto della directory 
   */

  check_type(&statbuf, c);
  
  /* Puntatore alla fine del path */
  ptr = fullpath + strlen(fullpath);
  *ptr++ = '/'; 
  *ptr = '\0';
  
  /* Apri la directory */
  /* ... */

  /* Cicla sui file della directory */
  while (/*... */) { 
	/* Ignora '.' e '..' */
	/* ... */

	/* Appendi il nome al path */
	/* ... */

	if ((ret = dopath(fullpath, c))) {
	  /* E' avvenuto un errore, esci dal ciclo while */
	  break; 
	}
  }
  
  /* Ripristina il path */
  *(ptr - 1) = '\0';

  /* Chiudi la directory */
  /* ... */

  return (ret); 
}

static void check_type(const struct stat *stat, struct counter *c) {
  switch (stat->st_mode & S_IFMT) { 
  case S_IFREG:
	c->reg++;
	break; 
  case S_IFDIR:
	c->dir++;
	break;
  case S_IFBLK: 
	c->blk++;
	break;
  case S_IFCHR: 
	c->chr++;
	break;
  case S_IFIFO: 
	c->fifo++;
	break;
  case S_IFLNK: 
	c->sym++;
	break;
  case S_IFSOCK: 
	c->sock++;
	break;
  default:
	return;
  }
  
  /* Incrementa il totale */
  c->tot++;
}
